//
//  Copyright (C) 2003,  . All Rights Reserved.
//
//  PROJECT:    Performance Counter Sample
//
//  MODULE:     safearray.h
//
//  PURPOSE:    -  SAFEARRAY. , 
//              . -   ,
//                   "RSDN\"
//
//  AUTHOR(S):   
//
//  PLATFORMS:  Windows 9x/NT/2000/XP
//

#ifndef SAFEARRAY_H
#define SAFEARRAY_H

template <typename T>
struct Type2Type
{
    typedef T OriginalType;
    Type2Type() {};
};

template<class T> class GetVT                 {public: enum {vt = VT_RECORD};};

template<>        class GetVT<long>           {public: enum {vt = VT_I4};};
template<>        class GetVT<int>            {public: enum {vt = VT_INT};};
template<>        class GetVT<short>          {public: enum {vt = VT_I2};};
template<>        class GetVT<char>           {public: enum {vt = VT_I1};};

template<>        class GetVT<unsigned long>  {public: enum {vt = VT_UI4};};
template<>        class GetVT<unsigned int>   {public: enum {vt = VT_UINT};};
template<>        class GetVT<unsigned short> {public: enum {vt = VT_UI2};};
template<>        class GetVT<unsigned char>  {public: enum {vt = VT_UI1};};

template<>        class GetVT<float>          {public: enum {vt = VT_R4};};
template<>        class GetVT<double>         {public: enum {vt = VT_R8};};

template<>        class GetVT<BSTR>           {public: enum {vt = VT_BSTR};};
template<>        class GetVT<VARIANT>        {public: enum {vt = VT_VARIANT};};
//  ,   short,  
template<>        class GetVT<Type2Type<VARIANT_BOOL> > {public: enum {vt = VT_BOOL};};
//  ,   double  
template<>        class GetVT<Type2Type<DATE> >         {public: enum {vt = VT_DATE};};


template<class T, int VT = GetVT<T>::vt>
class CComSafeVector
{
public:

    CComSafeVector() : array(NULL), data(NULL), lbound(0), ubound(0) {}

    CComSafeVector(const CComSafeVector<T, VT>& csv)
    {
        if (csv.array)
        {
            SafeArrayCopy(csv.array, &array);
            BoundsChanged();
        }
        else
        {
            array = NULL;
            lbound = 0;
            ubound = 0;
        }

        data = NULL;
    }

    ~CComSafeVector()
    {
        Destroy();
    }

    operator const SAFEARRAY*() const
    {
        return array;
    }

	SAFEARRAY** operator&()
	{
		ATLASSERT(array == NULL);
		return &array;
	}

    ////////////////////////////////////////////////////////////////
    //
    //  
    //
    ////////////////////////////////////////////////////////////////

    bool Create(int count, int low = 0)
    {
        ATLASSERT(array == NULL);

        SAFEARRAYBOUND bound = {count, low};

        array = SafeArrayCreate(VT, 1, &bound);

        if (NULL == array)
        {
            return false;
        }

        lbound = low;
        ubound = low + count - 1;

        return true;
    }

    HRESULT CreateEx(
                const GUID& guid, 
                const GUID& libid, 
                ULONG verMaj, ULONG verMin, 
                LCID lcid, 
                int count, int low = 0)
    {
        ATLASSERT(array == NULL);

        HRESULT hr;
        CComPtr<IRecordInfo> pRecInfo;

        hr = GetRecordInfoFromGuids(libid, verMaj, verMin, lcid, guid, &pRecInfo);

        if (FAILED(hr))
        {
            return hr;
        }

        SAFEARRAYBOUND bound = {count, low};
    
        array = SafeArrayCreateVectorEx(VT_RECORD, low, count, pRecInfo);

        if (NULL == array)
        {
            return E_OUTOFMEMORY;
        }

        lbound = low;
        ubound = low + count - 1;

        return S_OK;
    }

    ////////////////////////////////////////////////////////////////
    //
    //  
    //
    ////////////////////////////////////////////////////////////////

    void Destroy()
    {
        if (array)
        {
            UnaccessData();
            SafeArrayDestroy(array);
            array = NULL;
        }        
    }
    
    ////////////////////////////////////////////////////////////////
    //
    // Attach/Detach
    //
    ////////////////////////////////////////////////////////////////
    
    void Attach(SAFEARRAY* pa)
    {
        Destroy();
        array = pa;

        BoundsChanged();    
    }

    SAFEARRAY* Detach()
    {
        UnaccessData();
        SAFEARRAY* pa = array;
        array = NULL;
        return pa;
    }

    ////////////////////////////////////////////////////////////////
    //
    //     C-
    //
    ////////////////////////////////////////////////////////////////

    void AccessData() const
    {
        ATLASSERT(array);

        if (data) return;
        SafeArrayAccessData(array, (void**)&data);
    }

    void UnaccessData() const
    {
        if (!array) return;
        if (!data) return;

        SafeArrayUnaccessData(array);
        data = NULL;
    }

    long size() const  
    {
        return ubound - lbound + 1;
    }
    
    const T& operator[](int i) const
    {
        ATLASSERT(array);
        ATLASSERT(data);
        ATLASSERT(i < ubound - lbound + 1);    

        return data[i];
    }

    T& operator[](int i)
    {
        ATLASSERT(array);
        ATLASSERT(data);
        ATLASSERT(i < ubound - lbound + 1);    

        return data[i];
    }

    ////////////////////////////////////////////////////////////////
    //
    //       
    //
    ////////////////////////////////////////////////////////////////

    long GetLBound() const  
    {
        return lbound;
    }

    long GetUBound() const 
    {
        return ubound;
    }

    VARTYPE GetRealVartype() const
    {
        ATLASSERT(array);

        VARTYPE t;
        SafeArrayGetVartype(array, &t);
        return t;
    }

    int GetRealDims() const
    {
        ATLASSERT(array);

        return SafeArrayGetDim(array);
    }

    void BoundsChanged()
    {
        ATLASSERT(array);

        SafeArrayGetLBound(array, 1, &lbound);
        SafeArrayGetUBound(array, 1, &ubound);    
    }

    SAFEARRAY* get()
    {
        return array;
    }

private:

    SAFEARRAY*   array;
    mutable T*   data;
    long         lbound;
    long         ubound;
};

#endif //SAFEARRAY_H
