//
//  Copyright (C) 2003,  . All Rights Reserved.
//
//  PROJECT:    Performance Counter Sample
//
//  MODULE:     counter.cpp
//
//  PURPOSE:    ,  .   
//                   
//               
//
//  AUTHOR(S):   
//
//  PLATFORMS:  Windows NT/2000/XP
//

#include <windows.h>
#include "perf_objects.h"

namespace perf
{
	//
	//  
	union CounterTypeMask
	{
		CounterTypeMask(DWORD t) {dw = t;}
		CounterTypeMask(const CounterTypeMask& tm) {dw = tm.dw;}

		DWORD dw;
		struct
		{
			 unsigned int reserved1 : 8;
			 unsigned int size      : 2; //   , 
			 unsigned int type      : 2; //  , 
			 unsigned int reserved2 : 4; 
			 unsigned int subtype   : 4; //  , 
			 unsigned int time      : 2; //  , 
			 unsigned int calc      : 6; //    , 
			 unsigned int display   : 4; //  , 
		};

		void select_size()    {char c = size;    dw = 0; size    = c;}
		void select_type()    {char c = type;    dw = 0; type    = c;}
		void select_subtype() {char c = subtype; dw = 0; subtype = c;}
		void select_time()    {char c = time;    dw = 0; time    = c;}
		void select_calc()    {char c = calc;    dw = 0; calc    = c;}
		void select_display() {char c = display; dw = 0; display = c;}
	};

	DWORD UpTo4(DWORD v)
	{
		if (v % 4)
		{
			v += 4;
			v -= v % 4;
		}

		return v;
	}
	
	int base_name_index = 0;
	int base_help_index = 0;
}

///////////////////////////////////////////////////////////////////
//
// 
//
///////////////////////////////////////////////////////////////////

//  
//  FUNCTION:     counter_definition(DWORD, DWORD, DWORD) 
//
//  PARAMETERS:   [IN] offset -     
//                [IN] type   -  
//                [IN] size   -  
//
//  RETURN VALUE: 
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:      ,   . 
//
//  COMMENTS:     
// 
perf::counter_definition::counter_definition(DWORD offset, DWORD type, DWORD size) 
		: _name_offset(offset), _type(type) 
{
	CounterTypeMask t = _type;

	t.select_size();

	switch (t.dw)
	{
	case PERF_SIZE_DWORD: _size = sizeof(DWORD); break;
	case PERF_SIZE_LARGE: _size = sizeof(LARGE_INTEGER); break;
	case PERF_SIZE_ZERO:  _size = 0; break;
	default:              _size = UpTo4(size);
	}
}

//  
//  FUNCTION:     void set_size(DWORD) 
//
//  PARAMETERS:   [IN] s -  
//
//  RETURN VALUE: 
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:       ,   
//
//  COMMENTS:     
// 
void perf::counter_definition::set_size(DWORD s)
{
	CounterTypeMask t = _type;

	t.select_size();

	if (t.dw == PERF_SIZE_VARIABLE_LEN)
	{
		_size = UpTo4(s);
	}
}

//  
//  FUNCTION:     void fill_structure(PERF_COUNTER_DEFINITION*) 
//
//  PARAMETERS:   [OUT] pcd - 
//
//  RETURN VALUE: 
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:         
//
//  COMMENTS:     
// 
void perf::counter_definition::fill_structure(PERF_COUNTER_DEFINITION* pcd) const
{
	ZeroMemory(pcd, sizeof(*pcd));

	pcd->ByteLength            = sizeof(*pcd);
	pcd->CounterNameTitleIndex = base_name_index + _name_offset;
	pcd->CounterHelpTitleIndex = base_help_index + _name_offset;
	pcd->CounterSize           = _size;
	pcd->CounterType           = _type;
	pcd->CounterOffset         = _data_offset;
}

/////////////////////////////////////////////////////////