//
//  Copyright (C) 2003,  . All Rights Reserved.
//
//  PROJECT:    Performance Counter Sample
//
//  MODULE:     perf_objects.h
//
//  PURPOSE:       perf     
//               .
//
//  AUTHOR(S):   
//
//  PLATFORMS:  Windows NT/2000/XP
//

#ifndef PERF_OBJECTS_H
#define PERF_OBJECTS_H

#include <vector>

#ifdef _DEBUG
#include <crtdbg.h>
#else
#define _ASSERT(x) 
#endif

namespace perf
{
	//
	//    
	extern int base_name_index;
	extern int base_help_index;

	//
	//     8
	DWORD UpTo8(DWORD v);

	//
	// .   counter.cpp
	class counter_definition
	{
	public:
		counter_definition(DWORD offset, DWORD type, DWORD size = 0);

		DWORD get_name_offset() const {return _name_offset;}
		DWORD get_type()        const {return _type;}
		DWORD get_size()        const {return _size;}
		DWORD get_offset()      const {return _data_offset;}

		void set_size(DWORD s); 
		void set_offset(DWORD o) const {_data_offset = o;}
		
		void fill_structure(PERF_COUNTER_DEFINITION* pcd) const;

		static int size() {return sizeof(PERF_COUNTER_DEFINITION);}

	private:

		DWORD _name_offset;
		DWORD _type;
		DWORD _size;

		mutable DWORD _data_offset;
	};

	//
	//  "Global"  "Costly"  .   object_abstract.cpp
	bool find_Global(const WCHAR* str);
	bool find_Costly(const WCHAR* str);

	//
	//    .     .
	//   object_abstract.cpp
	class object_type_abstract 
	{
	public:
		object_type_abstract(DWORD offset);

		virtual void cleanup();

		//
		//    
		//

		DWORD get_name_offset() const;
		bool find_object(const WCHAR* str) const;

		//
		//   
		// 

		const int                 get_counters_count() const;
		const counter_definition& get_counter(int i) const;

		void add_counter(DWORD offset, DWORD type, DWORD size = 0);
		void del_counter(int i);
		void set_counter_size(int i, DWORD size);

		void calculate_offsets() const;

		//
		//   
		//

		int get_definition_length() const;
		int get_data_length() const;

		virtual int get_total_length() const = 0;
		virtual int get_instances_count() const = 0;

		//
		//    
		//

		void set_buffer(void* buffer);

		//        
		void write_header();

	protected:

		//       
		int  get_counter_data_offset(int i) const;
		//    . 
		void fill_structure(PERF_OBJECT_TYPE* pot) const;

		void write_buffer(int offset, const void* data, int size)
		{
			_ASSERT(_buffer);
			memcpy((char*)_buffer + offset, data, size); 
		}

		const void* read_buffer(int offset) const
		{
			_ASSERT(_buffer);
			return (char*)_buffer + offset;
		}

	private:

		DWORD _name_offset;
		void* _buffer;
		std::vector<counter_definition> _counters;

		mutable bool  _calculated;
	};

	//
	// "" .   object_single.cpp
	class object_type_single : public object_type_abstract
	{
	public:
		object_type_single(DWORD offset) : object_type_abstract(offset) {}

		virtual int get_total_length() const;
		virtual int get_instances_count() const {return PERF_NO_INSTANCES;}

		void write_counter32(int i, int value);
		void write_counter64(int i, __int64 value);
		void write_counter(int i, const void* value, int size);
	};

	//
	// "" .   object_instance.cpp
	class object_type_instance : public object_type_abstract
	{
	public:
		object_type_instance(DWORD offset) : object_type_abstract(offset) {}

		int get_instance_with_out_name_length() const;
		int get_next_length(DWORD id) const;
		int get_next_length(const WCHAR* name) const;

		virtual int get_total_length() const;
		virtual int get_instances_count() const; 

		//
		//       
		void clear_all_instances();

		//
		//  
		void write_instance(DWORD id, DWORD parent_index = 0, DWORD parent_instance = 0);
		void write_instance(const WCHAR* name, DWORD parent_index = 0, DWORD parent_instance = 0);

		//
		//    
		void write_counter32(int inst, int i, int value);
		void write_counter64(int inst, int i, __int64 value);
		void write_counter(int inst, int i, const void* value, int size);

		//
		//    
		void write_last_counter32(int i, int value);
		void write_last_counter64(int i, __int64 value);
		void write_last_counter(int i, const void* value, int size);
	
	private:

		int get_instance_offset(int i) const;
		int get_instance_counter_data_offset(int inst, int i) const;

		struct instance_data
		{
			int index;
			int offset;

			instance_data() : index(-1), offset(0) {}
		};

		instance_data _last;
		mutable instance_data _last_use;
	};
}

#endif //PERF_OBJECTS_H