//
//  Copyright (C) 2003,  . All Rights Reserved.
//
//  PROJECT:    Performance Counter Sample
//
//  MODULE:     perf_impl.cpp
//
//  PURPOSE:     ,     
//              .   
//              
//
//  AUTHOR(S):   
//
//  PLATFORMS:  Windows NT/2000/XP
//

#include "stdafx.h"
#include "offset.h"
#include "registry.h"
#include "./perf/perf_objects.h"
#include "./com/com_support.h"
#include "./com/safearray.h"
#include "./com/behemothserver.h"
#include "cache.h"
#include "perf_impl.h"

namespace perf_impl
{
	void parse_request(const WCHAR* str, bool* need_bs, bool* need_b);

	DWORD collect_behemoths_summary(void* data, DWORD size, DWORD* bytes);
	DWORD collect_behemoth(void* data, DWORD size, DWORD* bytes);
	
	perf::object_type_single   behemoths_summ(BEHEMOTHS_SUMM_OFFSET);
	perf::object_type_instance behemoth(BEHEMOTH_OFFSET);

	long open_succes = 0;
}

//////////////////////////////////////////////////////////////
//
//  
//
//////////////////////////////////////////////////////////////

//  
//  FUNCTION:     DWORD Open()
//
//  PARAMETERS:   
//
//  RETURN VALUE:  
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:       
//
//  COMMENTS:      
//
DWORD perf_impl::Open()
{
	if (open_succes)
	{
		//  dll    
		open_succes++;
		return ERROR_SUCCESS;
	}

	DWORD name;
	DWORD help;

	DWORD res = Registry::GetFirst(&name, &help);

	if (res != ERROR_SUCCESS)
	{
		//  . ,   .
		return res;
	}

	res = com_support::Init();

	if (res != ERROR_SUCCESS)
	{
		//  .
		return res;
	}

	//  
	perf::base_name_index = name;
	perf::base_help_index = help;

	//  
	behemoths_summ.add_counter(BEHEMOTHS_COUNT_OFFSET,  PERF_SIZE_DWORD | PERF_TYPE_NUMBER);
	behemoths_summ.add_counter(BEHEMOTHS_WEIGHT_OFFSET, PERF_SIZE_DWORD | PERF_TYPE_NUMBER);

	//  
	behemoth.add_counter(SEX_OFFSET,      PERF_SIZE_DWORD | PERF_TYPE_NUMBER);
	behemoth.add_counter(AGE_OFFSET,      PERF_SIZE_DWORD | PERF_TYPE_NUMBER);
	behemoth.add_counter(X_OFFSET,        PERF_SIZE_DWORD | PERF_TYPE_NUMBER);
	behemoth.add_counter(Y_OFFSET,        PERF_SIZE_DWORD | PERF_TYPE_NUMBER);
	behemoth.add_counter(WEIGHT_OFFSET,   PERF_SIZE_DWORD | PERF_TYPE_NUMBER);
	behemoth.add_counter(VOLUME_OFFSET,   PERF_SIZE_DWORD | PERF_TYPE_NUMBER);
	behemoth.add_counter(BIRTHDAY_OFFSET, PERF_SIZE_LARGE | PERF_TYPE_NUMBER);
	
	open_succes = 1;
	return ERROR_SUCCESS;
}

//  
//  FUNCTION:     void Close()
//
//  PARAMETERS:   
//
//  RETURN VALUE: 
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:       
//
//  COMMENTS:      
//
void perf_impl::Close()
{
	open_succes--;

	if (open_succes == 0)
	{
		FinalClose();
	}
}

//  
//  FUNCTION:     void FinalClose()
//
//  PARAMETERS:   
//
//  RETURN VALUE: 
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:        
//
//  COMMENTS:      
//
void perf_impl::FinalClose()
{
	cache::Cleanup();
	com_support::Cleanup();

	behemoths_summ.cleanup();
	behemoth.cleanup();

	open_succes = 0;	
}

//  
//  FUNCTION:     DWORD Collect(const WCHAR* , void* , DWORD , DWORD* , DWORD* )
//
//  PARAMETERS:   [IN]     str     -  
//                [IN/OUT] data    -   
//                [IN]     size    -  
//                [OUT]    bytes   -   
//                [OUT]    objects -   
//
//  RETURN VALUE:  
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:            :) 
//
//  COMMENTS:     
//
DWORD perf_impl::Collect(
					const WCHAR* request, 
					void* data, 
					DWORD size, 
					DWORD* bytes, 
					DWORD* objects)
{
	*bytes = 0;
	*objects = 0;

	if (!open_succes)
	{
		//  
		return E_FAIL;
	}

	bool need_behemoths_summ = false;
	bool need_behemoth = false;

	//  
	parse_request(request, &need_behemoths_summ, &need_behemoth);

	if (!need_behemoths_summ && !need_behemoth)
	{
		//   
		return ERROR_SUCCESS;
	}

	//
	//     
	//

	int minimal_size = 0;

	if (need_behemoths_summ)
	{
		minimal_size += behemoths_summ.get_total_length();
	}

	if (need_behemoth)
	{
		minimal_size += behemoth.get_definition_length();
	}

	if (size < minimal_size)
	{
		//     
		return ERROR_MORE_DATA;
	}

	//
	//  
	//

	//      need_behemoth   
	//  GetArray    GetSummary   
	if (need_behemoth)
	{
		HRESULT hr = cache::GetArray();

		if (FAILED(hr))
		{
			return hr;
		}
	}

	if (need_behemoths_summ)
	{
		HRESULT hr = cache::GetSummary();

		if (FAILED(hr))
		{
			return hr;
		}
	}

	//
	//   
	//

	if (need_behemoths_summ)
	{
		DWORD writen;
		DWORD res = collect_behemoths_summary(data, size, &writen);

		if (res != ERROR_SUCCESS)
		{
			return res;
		}

		*objects += 1;
		*bytes   += writen;
	}

	if (need_behemoth)
	{
		DWORD writen;
		DWORD res = collect_behemoth((char*)data + *bytes, size - *bytes, &writen);

		if (res != ERROR_SUCCESS)
		{
			*bytes   = 0;
			*objects = 0;
			return res;
		}

		*objects += 1;
		*bytes   += writen;
	}

	return ERROR_SUCCESS;
}

//////////////////////////////////////////////////////////////
//
// C 
//
//////////////////////////////////////////////////////////////

//  
//  FUNCTION:     void parse_request(const WCHAR*, bool*, bool*)
//
//  PARAMETERS:   [IN]  str     -  
//                [OUT] need_bs -    behemoths_summ
//                [OUT] need_b  -    behemoth
//
//  RETURN VALUE: 
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:        , ,     
//
//  COMMENTS:     
//
void perf_impl::parse_request(const WCHAR* str, bool* need_bs, bool* need_b)
{
	*need_bs = false;
	*need_b  = false;

	if ((str == NULL) || (lstrlenW(str) == 0) || perf::find_Global(str))
	{
		*need_bs = true;
		*need_b  = true;
		return;
	}

	if (behemoths_summ.find_object(str))
	{
		*need_bs = true;
	}

	if (behemoth.find_object(str))
	{
		*need_b = true;
	}
}

//  
//  FUNCTION:     DWORD collect_behemoths_summary(void*, DWORD, DWORD*)
//
//  PARAMETERS:   [IN/OUT] data  -   
//                [IN]     size  -  
//                [OUT]    bytes -   
//
//  RETURN VALUE:  
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:           behemoths_summ
//
//  COMMENTS:     
//
DWORD perf_impl::collect_behemoths_summary(void* data, DWORD size, DWORD* bytes)
{
	if (size < behemoths_summ.get_total_length())
	{
		//  
		return ERROR_MORE_DATA;
	}

	behemoths_summ.set_buffer(data);

	behemoths_summ.write_header();
	behemoths_summ.write_counter32(0, cache::GetCount());
	behemoths_summ.write_counter32(1, cache::GetWeight());

	*bytes = behemoths_summ.get_total_length();

	return ERROR_SUCCESS;
}

//  
//  FUNCTION:     DWORD collect_behemoth(void*, DWORD, DWORD*)
//
//  PARAMETERS:   [IN/OUT] data  -   
//                [IN]     size  -  
//                [OUT]    bytes -   
//
//  RETURN VALUE:  
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:           behemoth
//
//  COMMENTS:     
//
DWORD perf_impl::collect_behemoth(void* data, DWORD size, DWORD* bytes)
{
	//   
	behemoth.clear_all_instances();

	if (size < behemoth.get_definition_length() 
					+ behemoth.get_instance_with_out_name_length() * cache::GetCount())
	{
		//     
		return ERROR_MORE_DATA;
	}

	behemoth.set_buffer(data);

	//   -
	DWORD behemoth_index = perf::base_name_index + behemoth.get_name_offset();

	for (int i = 0; i < cache::GetCount(); i++)
	{
		if (size < behemoth.get_next_length(cache::Get(i).name))
		{
			//  
			return ERROR_MORE_DATA;
		}

		//  
		behemoth.write_instance(cache::Get(i).name, behemoth_index, cache::Get(i).mother);

		//  
		behemoth.write_last_counter32(0, cache::Get(i).sex);
		behemoth.write_last_counter32(1, cache::Get(i).age);
		behemoth.write_last_counter32(2, cache::Get(i).x);
		behemoth.write_last_counter32(3, cache::Get(i).y);
		behemoth.write_last_counter32(4, cache::Get(i).weight);
		behemoth.write_last_counter32(5, cache::Get(i).volume);
		behemoth.write_last_counter64(6, cache::Get(i).birthday);
	}

	//  
	behemoth.write_header();

	*bytes = behemoth.get_total_length();

	return ERROR_SUCCESS;
}

///////////////////////////////////////////////////////