//
//  Copyright (C) 2003,  . All Rights Reserved.
//
//  PROJECT:    Performance Counter Sample
//
//  MODULE:     registry.cpp
//
//  PURPOSE:     ,    
//
//  AUTHOR(S):   
//
//  PLATFORMS:  Windows 9x/NT/2000/XP
//

#include "stdafx.h"
#include "registry.h"

namespace Registry
{
	const TCHAR* perfKey   = TEXT("System\\CurrentControlSet\\Services\\Behemoth\\Performance");
	const TCHAR* firstName = TEXT("First Counter");
	const TCHAR* firstHelp = TEXT("First Help");

	int GetDWORD(HKEY hKey, const TCHAR* value, DWORD* data);
	int SetString(HKEY hKey, const TCHAR* value, const TCHAR* data);
}

//////////////////////////////////////////////////////////////////////
//
//  
//
//////////////////////////////////////////////////////////////////////

//  
//  FUNCTION:     int DllRegister(HMODULE)
//
//  PARAMETERS:   [IN] hModule -   dll
//
//  RETURN VALUE:  
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:       dll,  dll   
//
//  COMMENTS:     
//
int Registry::DllRegister(HMODULE hModule)
{
	LONG res;
	HKEY hPerf;

	res = RegCreateKeyEx(
			HKEY_LOCAL_MACHINE,
			perfKey,
			0,
			0,
			0,
			KEY_SET_VALUE,
			NULL,
			&hPerf,
			NULL);

	if (res != ERROR_SUCCESS) 
	{
		return res;
	}

	TCHAR str[_MAX_PATH];
	GetModuleFileName(hModule, str, sizeof(str)/sizeof(str[0]));

	SetString(hPerf, TEXT("Library"), str); 
	SetString(hPerf, TEXT("Open"),    TEXT("Open")); 
	SetString(hPerf, TEXT("Close"),   TEXT("Close")); 
	SetString(hPerf, TEXT("Collect"), TEXT("Collect")); 

	RegCloseKey(hPerf);

	return ERROR_SUCCESS;
}

//  
//  FUNCTION:     int DllUnregister()
//
//  PARAMETERS:   
//
//  RETURN VALUE:  
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:         dll  
//
//  COMMENTS:     
//
int Registry::DllUnregister()
{
	RegDeleteKey(HKEY_LOCAL_MACHINE, perfKey);

	TCHAR str[_MAX_PATH];

	lstrcpy(str, perfKey);

	int i = lstrlen(str);

	while (str[i] != TEXT('\\')) i--;
	str[i] = 0;

	RegDeleteKey(HKEY_LOCAL_MACHINE, str);

	return ERROR_SUCCESS;
}

//  
//  FUNCTION:     int GetFirst(DWORD*, DWORD*)
//
//  PARAMETERS:   [OUT] name -   
//                [OUT] help -   
//
//  RETURN VALUE:  
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:           
//
//  COMMENTS:     
//
int Registry::GetFirst(DWORD* name, DWORD* help)
{
	LONG res;
	HKEY hPerf;

	res = RegOpenKeyEx(
			HKEY_LOCAL_MACHINE,
			perfKey,
			0,
			KEY_QUERY_VALUE,
			&hPerf);

	if (res != ERROR_SUCCESS) 
	{
		return res;
	}

	res = GetDWORD(hPerf, firstName, name);

	if (res != ERROR_SUCCESS) 
	{
		RegCloseKey(hPerf);
		return res;
	}

	res = GetDWORD(hPerf, firstHelp, help);

	RegCloseKey(hPerf);

	if (res != ERROR_SUCCESS) 
	{
		return res;
	}

	return ERROR_SUCCESS;
}

//////////////////////////////////////////////////////////////
//
//  
//
//////////////////////////////////////////////////////////////

//  
//  FUNCTION:     int GetDWORD(HKEY , const TCHAR* , DWORD* )
//
//  PARAMETERS:   [IN]  hKey  -   
//                [IN]  value - 
//                [OUT] data  - 
//
//  RETURN VALUE:  
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:          REG_DWORD
//
//  COMMENTS:     
//
int Registry::GetDWORD(HKEY hKey, const TCHAR* value, DWORD* data)
{
	LONG res;
	DWORD type = REG_DWORD;
	DWORD size = sizeof(DWORD);

	res = RegQueryValueEx(hKey, value, 0, &type, (BYTE*)data, &size);

	if (res != ERROR_SUCCESS) 
	{
		return res;
	}

	if (type != REG_DWORD)
	{
		return ERROR_INVALID_DATA;
	}

	return ERROR_SUCCESS;
}

//  
//  FUNCTION:     int SetString(HKEY , const TCHAR* , const TCHAR* )
//
//  PARAMETERS:   [IN] hKey  -   
//                [IN] value - 
//                [IN] data  - 
//
//  RETURN VALUE:  
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:          REG_SZ
//
//  COMMENTS:     
//
int Registry::SetString(HKEY hKey, const TCHAR* value, const TCHAR* data)
{
	return RegSetValueEx(
				hKey, 
				value, 
				0, 
				REG_SZ, 
				(const BYTE*)data, 
				(lstrlen(data) + 1) * sizeof(data[0]));
}

/////////////////////////////////////////////////////////////