#include "stdwtl.h"
#include "behemothmanager.h"
#include "behemothtree.h"

namespace
{
	// ,   
	const int image_x     = 16;
	const int image_y     = 16;
	const int image_count = 9;

	const int behemoth_m_img = 0;
	const int behemoth_w_img = 1;
	const int sex_img        = 2;
	const int age_img        = 3;
	const int x_img          = 4;
	const int y_img          = 5;
	const int weight_img     = 6;
	const int volume_img     = 7;
	const int birthday_img   = 8;
};

//////////////////////////////////////////////////////////////////////
//
//  
//
//////////////////////////////////////////////////////////////////////

CBehemothsTree::CBehemothsTree() 
{
}

CBehemothsTree::~CBehemothsTree()
{
	_image.Destroy();
}

bool CBehemothsTree::Create(HWND parent)
{
	CreateImageList();

	HWND hTree;

	hTree = CTreeViewCtrl::Create( 
						parent,
						NULL, 
						NULL, 
						WS_CHILD | WS_VISIBLE | WS_BORDER
					  | TVS_HASBUTTONS | TVS_DISABLEDRAGDROP | TVS_HASLINES  
					  | TVS_LINESATROOT | TVS_SHOWSELALWAYS,
						WS_EX_CLIENTEDGE);

	if (NULL == hTree)
	{
		return false;
	}

	SetImageList(_image, TVSIL_NORMAL);

	return true;
}

void CBehemothsTree::Clear()
{
	SendMessage(m_hWnd, WM_SETREDRAW, FALSE, 0);
	DeleteAllItems();
	SendMessage(m_hWnd, WM_SETREDRAW, TRUE, 0);
}

//
//   
//
void CBehemothsTree::Fill(int size, const Behemoth* data)
{
	Clear();

	SendMessage(m_hWnd, WM_SETREDRAW, FALSE, 0);

	std::vector<HTREEITEM> item;

	item.resize(size, 0);

	int child_index = -1;
	int i = 0;

	while (i < size)
	{
		if (item[i] != 0)
		{
			//   
			i++;
			continue;
		}

		if ((data[i].mother < 0) || (item[data[i].mother] != 0))
		{
			HTREEITEM hMother;

			if (data[i].mother < 0) 
			{
				//    
				hMother = TVI_ROOT;
			}
			else
			{
				//   
				hMother = item[data[i].mother];
			}

			item[i] = InsertBehemoth(hMother, &(data[i]), i);

			if (child_index == -1)
			{
				i++;
			}
			else
			{
				i = child_index;
				child_index = -1;
			}

			continue;
		}

		if (child_index == -1)
		{
			child_index = i;
		}

		i = data[i].mother;
	}

	SendMessage(m_hWnd, WM_SETREDRAW, TRUE, 0);
}

//
//     
int CBehemothsTree::GetCurrentItem()
{
	HTREEITEM hCurr = GetSelectedItem();

	if (hCurr == NULL) 
	{
		return -1;
	}

	TVITEM item = {0};

	item.mask  = TVIF_HANDLE | TVIF_PARAM | TVIF_IMAGE;
	item.hItem = hCurr;

	GetItem(&item);

	return item.lParam;
}

///////////////////////////////////////////////////////////////
//
//  
//
///////////////////////////////////////////////////////////////

bool CBehemothsTree::CreateImageList()
{
	_image.Create(image_x, image_y, 0, image_count, 0);

	AddBitmap(IDB_BEHEMOTH_M);
	AddBitmap(IDB_BEHEMOTH_W);
	AddBitmap(IDB_SEX);
	AddBitmap(IDB_AGE);
	AddBitmap(IDB_X);
	AddBitmap(IDB_Y);
	AddBitmap(IDB_WEIGHT);
	AddBitmap(IDB_VOLUME);
	AddBitmap(IDB_BIRTHDAY);

	return true;
}

bool CBehemothsTree::AddBitmap(int id)
{
	HBITMAP hBmp = LoadBitmap(
						_Module.GetResourceInstance(), 
						MAKEINTRESOURCE(id));
	
	_image.Add(hBmp);

	DeleteObject(hBmp);

	return true;
}

HTREEITEM CBehemothsTree::InsertBehemoth(HTREEITEM hMother, const Behemoth* obj, int param)
{
	HTREEITEM hObject;

	if (obj->sex == Behemoth_man)
	{
		hObject = InsertW(hMother, obj->name, behemoth_m_img, param);
		Insert(hObject, TEXT("man"), sex_img, param);
	}
	else if (obj->sex == Behemoth_woman)
	{
		hObject = InsertW(hMother, obj->name, behemoth_w_img, param);
		Insert(hObject, TEXT("woman"), sex_img, param);
	}
	else
	{
		return NULL;
	}

	InsertInt(hObject, TEXT("Age"),    obj->age,    age_img,    param);
	InsertInt(hObject, TEXT("X"),      obj->x,      x_img,      param);
	InsertInt(hObject, TEXT("Y"),      obj->y,      y_img,      param);
	InsertInt(hObject, TEXT("Weight"), obj->weight, weight_img, param);
	InsertInt(hObject, TEXT("Volume"), obj->volume, volume_img, param);

	InsertDate(hObject, TEXT("Was born at"), obj->birthday, birthday_img, param);

	return hObject;
}

//
void CBehemothsTree::InsertInt(HTREEITEM parent, const TCHAR* name, int data, int img, int param)
{
	TCHAR str[100];

	wsprintf(str, TEXT("%s = %d"), name, data);

	Insert(parent, str, img, param);
}

void CBehemothsTree::InsertDate(HTREEITEM parent, const TCHAR* name, __int64 data, int img, int param)
{
	FILETIME ft = {0};

	memcpy(&ft, &data, sizeof(data));

	SYSTEMTIME st = {0};

	FileTimeToSystemTime(&ft, &st);

	TCHAR str[100];

	wsprintf(str, 
			TEXT("%s %04d.%02d.%02d %02d:%02d:%02d"), name, 
			st.wYear, st.wMonth, st.wDay, 
			st.wHour, st.wMinute, st.wSecond);

	Insert(parent, str, img, param);
}

//
//     
HTREEITEM CBehemothsTree::InsertW(HTREEITEM parent, const WCHAR* nameW, int img, int param)
{
#ifndef UNICODE
	
	int req = WideCharToMultiByte(CP_ACP, 0, nameW, -1, NULL, 0, 0, 0); 

	char* name = (char*) malloc(req + 1);
	name[0] = 0;

	WideCharToMultiByte(CP_ACP, 0, nameW, -1, name, req, 0, 0); 
	name[req] = 0;

	HTREEITEM res = Insert(parent, name, img, param);

	free(name);

	return res;

#else  // UNICODE

	return Insert(parent, nameW, img, param);

#endif // UNICODE
}

//
//     
HTREEITEM CBehemothsTree::Insert(HTREEITEM parent, const TCHAR* name, int img, int param)
{
	TV_INSERTSTRUCT item = {0};

	item.hParent = parent;
	
	//    
	item.hInsertAfter = TVI_LAST;

	item.item.mask           = TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE | TVIF_PARAM;
	item.item.pszText        = (TCHAR*)name;
	item.item.iImage         = img;
	item.item.iSelectedImage = img;
	item.item.lParam         = (LPARAM)param;

	return InsertItem(&item);
}


/////////////////////////////////////////////////////////