#include "stdwtl.h"
#include "behemothmanager.h"
#include "behemothview.h"

////////////////////////////////////////////////////////////////////
//
//  
//
////////////////////////////////////////////////////////////////////

HWND CBehemothsView::Create(HWND hParent) 
{
	RECT rc = {0};

	CBase::Create(hParent, rc);

	CDC display;

	display.CreateDC(TEXT("DISPLAY"), NULL, NULL, NULL);

	memdc.CreateCompatibleDC(display);

	bmp_size_x = GetSystemMetrics(SM_CXSCREEN);
    bmp_size_y = GetSystemMetrics(SM_CYSCREEN);

	bitmap.CreateCompatibleBitmap(display, bmp_size_x, bmp_size_y);

	SelectObject(memdc, bitmap);
	RedrawBitmap(0, NULL, -1);
	
	return m_hWnd;
}

void CBehemothsView::OnPaint(HDC)
{
	PAINTSTRUCT ps;
	HDC hdc = BeginPaint(&ps);
		
	BitBlt(hdc, 0, 0, wnd_size_x, wnd_size_y, memdc, 0, 0, SRCCOPY);

	EndPaint(&ps);
}

LRESULT CBehemothsView::OnErase(HDC hdc)
{
	return 1;
}

void CBehemothsView::OnSize(UINT, CSize s)
{
	wnd_size_x = s.cx;
	wnd_size_y = s.cy;
}

void CBehemothsView::Redraw(int size, const Behemoth* data, int selected)
{
	RedrawBitmap(size, data, selected);
	InvalidateRect(NULL, TRUE);
}

////////////////////////////////////////////////////////////////////
//
//  
//
////////////////////////////////////////////////////////////////////

void CBehemothsView::RedrawBitmap(int size, const Behemoth* data, int selected)
{
	RECT rc = {0};

	rc.right  = bmp_size_x;
	rc.bottom = bmp_size_y;

	FillRect(memdc, &rc, GetBackgroundBrush());

	//
	//  
	//

	for (int i = 0; i < size; i++)
	{
		if (data[i].sex != Behemoth_man) continue;

		CalculateRect(data[i], &rc);

		if (rc.bottom == rc.top)
		{
			SetPixel(memdc, rc.top, rc.left, RGB(100, 100, 255));
		}
		else
		{
			FillRect(memdc, &rc, GetManBrush());
		}
	}

	//
	//  
	//

	HBRUSH hOldB = (HBRUSH)SelectObject(memdc, GetWomanBrush());
	HPEN   hOldP = (HPEN)  SelectObject(memdc, GetWomanPen());

	for (i = 0; i < size; i++)
	{
		if (data[i].sex != Behemoth_woman) continue;

		CalculateRect(data[i], &rc);

		if (rc.bottom == rc.top)
		{
			SetPixel(memdc, rc.top, rc.left, RGB(255, 100, 100));
		}
		else
		{
			Ellipse(memdc, rc.left, rc.top, rc.right, rc.bottom);
		}
	}

	SelectObject(memdc, hOldB);
	SelectObject(memdc, hOldP);

	if (selected < 0)
	{
		return;
	}

	//
	//  
	//

	CalculateRect(data[selected], &rc);

	if (rc.bottom == rc.top)
	{
		SetPixel(memdc, rc.top, rc.left, RGB(100, 255, 100));
		return;
	}

	if (data[selected].sex == Behemoth_man)
	{
		FillRect(memdc, &rc, GetSelectBrush());
		return;
	}

	hOldB = (HBRUSH)SelectObject(memdc, GetSelectBrush());
	hOldP = (HPEN)  SelectObject(memdc, GetSelectPen());

	Ellipse(memdc, rc.left, rc.top, rc.right, rc.bottom);

	SelectObject(memdc, hOldB);
	SelectObject(memdc, hOldP);
}

void CBehemothsView::CalculateRect(const Behemoth& data, RECT* rc)
{
	int edge = data.volume / 300;

	rc->left = 4 * data.x - edge / 2;
	rc->top  = 4 * data.y - edge / 2;

	rc->right  = rc->left + edge;
	rc->bottom = rc->top  + edge;
}

HBRUSH CBehemothsView::GetBackgroundBrush()
{
	static CBrush b(CreateSolidBrush(RGB(0, 200, 255)));
	return b;
}

HBRUSH CBehemothsView::GetManBrush()
{
	static CBrush b(CreateSolidBrush(RGB(100, 100, 255)));
	return b;
}

HBRUSH CBehemothsView::GetWomanBrush()
{
	static CBrush b(CreateSolidBrush(RGB(255, 100, 100)));
	return b;
}

HPEN CBehemothsView::GetWomanPen()
{
	static CPen p(CreatePen(PS_SOLID, 1, RGB(255, 100, 100)));
	return p;
}

HBRUSH CBehemothsView::GetSelectBrush()
{
	static CBrush b(CreateSolidBrush(RGB(100, 255, 100)));
	return b;
}

HPEN CBehemothsView::GetSelectPen()
{
	static CPen p(CreatePen(PS_SOLID, 1, RGB(100, 255, 100)));
	return p;
}

////////////////////////////////////////////////////////////////////
