#include "stdafx.h"
#include "behemotharium.h"
#include <vector>

namespace Behemotharium
{
	const int aquarium_x = 100;
	const int aquarium_y = 100;

	const int born_age     = 180; 
	const int adult_age    = 360;
	const int decrepit_age = 1024;

	const int max_v = 10;	
	const int max_count = 100;

	class BehemothInternal
	{
	public:
		BehemothInternal() {name = 0;}
		~BehemothInternal() {}

		void Init(const WCHAR* n, int s, int m, int i);
		void Cleanup();

		void FriendDead(int n);

		bool Go();

		void Fill(Behemoth* b) const;

		bool IsGuy() const;
		bool IsReady() const;
		bool IsMatch(const BehemothInternal& b) const;

		int GetWeight() const {return weight;}

		void Conceive(const BehemothInternal& b);

	private:

		WCHAR* name;
		int mother;
		int sex;
		int x;
		int y;
		int weight;
		int volume;
		int age;
		__int64 birthday;

		int vx;
		int vy;
		int index;

		int baby_age;
	};

	DWORD CALLBACK Thread(LPVOID);

	void CreateRandom(int m);
	void CreateInternal(const WCHAR* name, int sex, int m);

	HANDLE hFinish = NULL;
	HANDLE hLock = NULL;
	HANDLE hThread = NULL;

	std::vector<BehemothInternal> behemoths;

	bool pause = false;
};

/////////////////////////////////////////////////////////////
//
//  
//
/////////////////////////////////////////////////////////////

void Behemotharium::Start()
{
	if (hThread) return;

	hFinish = CreateEvent(NULL, TRUE, FALSE, NULL);

	hLock = CreateMutex(NULL, FALSE, NULL);

	hThread = CreateThread(NULL, 0, Thread, 0, 0, 0);
}

void Behemotharium::Stop()
{
	if (!hThread) return;

	SetEvent(hFinish);
	
	if (WaitForSingleObject(hThread, 1000) != WAIT_OBJECT_0)
	{
		TerminateThread(hThread, 0);
	}

	CloseHandle(hFinish);
	CloseHandle(hThread);
	CloseHandle(hLock);

	hThread = NULL;
}

void Behemotharium::Pause()
{
	pause = true;
}

void Behemotharium::Continue()
{
	pause = false;
}

void Behemotharium::Lock()
{
	WaitForSingleObject(hLock, INFINITE);
}

void Behemotharium::Unlock()
{
	ReleaseMutex(hLock);
}

int Behemotharium::GetCount()
{
	return behemoths.size();
}

int Behemotharium::GetWeight()
{
	int weight = 0;

	for (int j = 0; j < behemoths.size(); j++)
	{
		weight += behemoths[j].GetWeight();
	}

	return weight;
}

void Behemotharium::Get(int i, Behemotharium::Behemoth* b)
{
	behemoths[i].Fill(b);
}

void Behemotharium::Kill(int i)
{
	for (int j = 0; j < behemoths.size(); j++)
	{
		behemoths[j].FriendDead(i);
	}

	behemoths.erase(behemoths.begin() + i);
}

void Behemotharium::Create(const WCHAR* name, int sex)
{
	CreateInternal(name, sex, -1);
}

/////////////////////////////////////////////////////////////
//
//  
//
/////////////////////////////////////////////////////////////

DWORD CALLBACK Behemotharium::Thread(LPVOID)
{
	int i;

	for (;;)
	{
		if (WaitForSingleObject(hFinish, 100) == WAIT_OBJECT_0)
		{
			//  
			break;
		}

		if (pause) continue;

		WaitForSingleObject(hLock, INFINITE);

		if (behemoths.size() == 0)
		{
			//    
			for (i = 0; i < 12; i++)
			{
				CreateRandom(-1);
			}
		}

		for (i = 0; i < behemoths.size(); i++)
		{
			if (!behemoths[i].Go())
			{
				//    ...
				Kill(i);
				i--;
			}
		}

		for (i = 0; i < behemoths.size(); i++)
		{
			if (!behemoths[i].IsGuy()) continue;

			for (int j = 0; j < behemoths.size(); j++)
			{
				if (!behemoths[j].IsReady()) continue;

				if (   behemoths[i].IsMatch(behemoths[j]) 
					&& behemoths[j].IsMatch(behemoths[i]))
				{
					//   
					behemoths[j].Conceive(behemoths[i]);
					break;
				}
			}
		}

		ReleaseMutex(hLock);
	}

	for (i = 0; i < behemoths.size(); i++)
	{
		behemoths[i].FriendDead(i);
	}

	behemoths.clear();

	return 0;
}

void Behemotharium::CreateRandom(int m)
{
	WCHAR name[16];

	int len = rand() % 11 + 5;

	name[0] = 'A' + rand() % 26;

	for (int i = 1; i < len; i++)
	{
		name[i] = 'a' + rand() % 26;
	}

	name[len] = 0;

	int sex = rand() % 2;

	if (sex == 0) 
	{
		sex = Behemoth::man;
	}
	else
	{
		sex = Behemoth::woman;
	}

	CreateInternal(name, sex, m);
}

void Behemotharium::CreateInternal(const WCHAR* name, int sex, int m)
{
	BehemothInternal bi;

	bi.Init(name, sex, m, behemoths.size());

	behemoths.push_back(bi);
}

/////////////////////////////////////////////////////////////
//
//   BehemothInternal
//
/////////////////////////////////////////////////////////////

void Behemotharium::BehemothInternal::Init(const WCHAR* n, int s, int m, int i)
{
	name = (WCHAR*) malloc((lstrlenW(n) + 1) * sizeof(WCHAR));

	lstrcpyW(name, n);

	sex = s;
	age = 0;
	mother = m;
	index = i;

	x = rand() % aquarium_x;
	y = rand() % aquarium_y;

	weight = rand() % 100 + 100;
	volume = rand() % 100 + 100;

	if (sex == Behemoth::man)
	{
		weight *= 1.5;
		volume *= 1.5;
	}

	SYSTEMTIME st;
	GetSystemTime(&st);
	SystemTimeToFileTime(&st, (FILETIME*)&birthday);

	vx = 0;
	vy = 0;

	baby_age = 0;
}

void Behemotharium::BehemothInternal::Cleanup()
{
	free(name);
}

void Behemotharium::BehemothInternal::FriendDead(int n)
{
	if (mother == n)
	{
		mother = -1;
	}
	else if (mother > n)
	{
		mother--;
	}

	if (index == n)
	{
		//    ..
		Cleanup();
	}
	else if (index > n)
	{
		index--;
	}
}

bool Behemotharium::BehemothInternal::Go()
{
	if (rand() + decrepit_age < age)
	{
		//     
		return false;
	}

	if ((sex == Behemoth::woman) && (baby_age < 0))
	{
		//  .
		baby_age++;

		volume += 5;
		weight += 5;

		if (baby_age == 0)
		{
			//   
			volume -= 5 * born_age;
			weight -= 5 * born_age;

			CreateRandom(index);
		}
	}

	vx += rand() % 5 - 2;
	vy += rand() % 5 - 2;

	int max_v_for_this_weight = max_v * 1000 / (1000 + weight);

	//   
	if (vx >  max_v_for_this_weight) vx =  max_v_for_this_weight;
	if (vx < -max_v_for_this_weight) vx = -max_v_for_this_weight;
	if (vy >  max_v_for_this_weight) vy =  max_v_for_this_weight;
	if (vy < -max_v_for_this_weight) vy = -max_v_for_this_weight;

	if (x + vx < 0)  vx = -x;
	if (y + vy < 0)  vy = -y;

	if (x + vx > aquarium_x)  vx = aquarium_x - x;
	if (y + vy > aquarium_y)  vy = aquarium_y - y;

	x += vx;
	y += vy;

	if (age < adult_age)
	{
		//   ...
		if (sex == Behemoth::woman) 
		{
			weight += rand() % 10;
			volume += rand() % 10;
		}
		else
		{
			weight += rand() % 15;
			volume += rand() % 15;
		}
	}

	age++;
	return true;
}

void Behemotharium::BehemothInternal::Fill(Behemoth* b) const
{
	b->name     = name;
	b->mother   = mother;
	b->sex      = sex;
	b->x        = x;
	b->y        = y;
	b->weight   = weight;
	b->volume   = volume;
	b->age      = age;
	b->birthday = birthday;
}

bool Behemotharium::BehemothInternal::IsGuy() const
{
	if (sex == Behemoth::woman)
	{
		return false;
	}

	return ((age >= adult_age) && (age < decrepit_age));
}

bool Behemotharium::BehemothInternal::IsReady() const
{
	if ((sex == Behemoth::woman) && (baby_age != 0))
	{
		//   
		return false;
	}

	if (GetCount() > max_count)
	{
		//     ..
		return false;
	}

	return ((age >= adult_age) && (age < decrepit_age));
}

bool Behemotharium::BehemothInternal::IsMatch(const BehemothInternal& b) const
{
	if (sex == b.sex)
	{
		//    ..
		return false;
	}

	if ((x - b.x < -10) || (x - b.x > 10) || (y - b.y < -10) || (y - b.y > 10))
	{
		//   ..
		return false;
	}

	//!!
	return true;
}

void Behemotharium::BehemothInternal::Conceive(const BehemothInternal& b)
{
	//  
	baby_age = -born_age;
}


//////////////////////////////////////////////////////////////