//
//  Copyright (C) 2003,  . All Rights Reserved.
//
//  PROJECT:    Performance Counter Sample
//
//  MODULE:     main.cpp
//
//  PURPOSE:     .      dll
//                  .
//
//  AUTHOR(S):   
//
//  PLATFORMS:  Windows NT/2000/XP
//

#include <windows.h>
#include "registry.h"

PERF_COUNTER_DEFINITION counter = {0};

typedef DWORD (CALLBACK* Open_type)(LPWSTR);
typedef DWORD (WINAPI*   Close_type)();
typedef DWORD (WINAPI*   Collect_type)(LPWSTR, LPVOID, LPDWORD, LPDWORD);

Open_type    fnOpen;
Close_type   fnClose;
Collect_type fnCollect;

HMODULE hOriginal = NULL;

//  v    8 
DWORD UpTo8(DWORD v)
{
	if (v % 8)
	{
		v += 8;
		v -= v % 8;
	}

	return v;
}

//  
//  FUNCTION:     DWORD Open(LPWSTR)
//
//  PARAMETERS:   [IN] lpDeviceNames - NULL
//
//  RETURN VALUE:  
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:       dll 
//
//  COMMENTS:     
//
extern "C" __declspec(dllexport) DWORD CALLBACK Open(LPWSTR lpDeviceNames)
{
	if (counter.CounterNameTitleIndex != 0)
	{
		return 0;
	}

	DWORD name;
	DWORD help;

	Registry::GetMy(&name, &help);

	//   
	counter.CounterNameTitleIndex = name;
	counter.CounterHelpTitleIndex = help;
	counter.CounterSize           = sizeof(DWORD);
	counter.CounterType           = PERF_SIZE_DWORD | PERF_TYPE_NUMBER;
	counter.CounterOffset         = 0;
	counter.ByteLength            = sizeof(counter);

	CHAR path[_MAX_PATH];
	CHAR open[_MAX_PATH];
	CHAR close[_MAX_PATH];
	CHAR collect[_MAX_PATH];

	//   
	Registry::GetOriginalDll(path, open, close, collect);

	// 
	hOriginal = LoadLibraryA(path);

	if (!hOriginal)
	{
		counter.CounterNameTitleIndex = 0;
		return GetLastError();
	}

	//  
	fnOpen    = (Open_type)   GetProcAddress(hOriginal, open); 
	fnClose   = (Close_type)  GetProcAddress(hOriginal, close); 
	fnCollect = (Collect_type)GetProcAddress(hOriginal, collect); 

	if (!fnOpen || !fnClose || !fnCollect)
	{
		FreeLibrary(hOriginal);
		hOriginal = NULL;
		counter.CounterNameTitleIndex = 0;
		return ERROR_INVALID_DATA;
	}

	return fnOpen(lpDeviceNames);
}

//  
//  FUNCTION:     DWORD Collect(LPWSTR, LPVOID *, LPDWORD, LPDWORD)
//
//  PARAMETERS:   [IN]     lpwszValue     -  
//                [IN/OUT] lppData        - 
//                [IN/OUT] lpcbBytes      - 
//                [OUT]    lpcObjectTypes -  
//
//  RETURN VALUE:  
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:          
//
//  COMMENTS:     
//
extern "C" __declspec(dllexport) DWORD WINAPI Collect(
											LPWSTR lpwszValue, 
											LPVOID *lppData, 
											LPDWORD lpcbBytes, 
											LPDWORD lpcObjectTypes)
{
	void* p = *lppData;
	DWORD size = *lpcbBytes;
	DWORD objects = 0;

	//     
	int res = fnCollect(lpwszValue, &p, &size, &objects);

	if ((res != 0) || (size == 0) || (objects == 0))
	{
		//  -    
		*lpcbBytes      = 0;
		*lpcObjectTypes = 0;
		return res;
	}

	//   

	PERF_OBJECT_TYPE* obj = (PERF_OBJECT_TYPE*)*lppData;

	for (int i = 0; i < objects; i++)
	{
		if (obj->ObjectNameTitleIndex == 2) break;

		obj = (PERF_OBJECT_TYPE*)((char*)obj + obj->TotalByteLength);
	}

	if (i == objects)
	{
		//    
		*lpcbBytes      = size;
		*lpcObjectTypes = objects;
		*lppData        = (char*)(*lppData) + size; 
		return ERROR_SUCCESS;
	}

	if (*lpcbBytes < size + UpTo8(sizeof(counter) + counter.CounterSize))
	{
		//       
		*lpcbBytes      = 0;
		*lpcObjectTypes = 0;
		return ERROR_MORE_DATA;
	}

	//     +   
	//  

	counter.CounterOffset = sizeof(PERF_COUNTER_BLOCK);
	PERF_COUNTER_DEFINITION* ctr;

	ctr = (PERF_COUNTER_DEFINITION*)((char*)obj + obj->HeaderLength);

	for (i = 0; i < obj->NumCounters; i++)
	{
		ctr = (PERF_COUNTER_DEFINITION*)((char*)ctr + ctr->ByteLength);

		int ctr_size = 0;

		switch (ctr->CounterType & PERF_SIZE_VARIABLE_LEN) 
		{
		case PERF_SIZE_DWORD: ctr_size = 4; break;
		case PERF_SIZE_LARGE: ctr_size = 8; break;
		case PERF_SIZE_ZERO:  ctr_size = 0; break;
		default:              ctr_size = ctr->CounterSize;
		}

		if (counter.CounterOffset < ctr->CounterOffset + ctr_size)
		{
			counter.CounterOffset = ctr->CounterOffset + ctr_size;
		}
	}

	int new_obj_size = obj->DefinitionLength + sizeof(counter) 
					+ counter.CounterOffset + counter.CounterSize;

	if (*lpcbBytes < size + UpTo8(new_obj_size) - obj->TotalByteLength)
	{
		//       
		*lpcbBytes      = 0;
		*lpcObjectTypes = 0;
		return ERROR_MORE_DATA;
	}

	//  ctr        
	
	//  
	memmove(
		(char*)ctr + sizeof(counter), 
		(char*)ctr,
		size - ((char*)ctr - (char*)(*lppData)));

	//   
	size += sizeof(counter);

	//  
	memcpy(ctr, &counter, sizeof(counter));

	//  ,   
	obj->NumCounters      += 1;
	obj->DefinitionLength += sizeof(counter);
	obj->TotalByteLength  += sizeof(counter);

	//  ,  

	PERF_COUNTER_BLOCK* bl = (PERF_COUNTER_BLOCK*)((char*)obj + obj->DefinitionLength);
	
	if (bl->ByteLength < counter.CounterOffset + counter.CounterSize)
	{
		//    
		int delta = UpTo8(counter.CounterOffset + counter.CounterSize) - bl->ByteLength;

		char* counter_begin = (char*)bl + counter.CounterOffset;
		
		//  
		memmove(
			counter_begin + delta, 
			counter_begin,
			size - (counter_begin - (char*)(*lppData)));

		//    
		bl->ByteLength       += delta;
		obj->TotalByteLength += delta;
		size                 += delta;
	}

	DWORD value = 123;

	//  
	memcpy((char*)bl + counter.CounterOffset, &value, sizeof(value));

	*lpcbBytes      = size;
	*lpcObjectTypes = objects;
	*lppData        = (char*)(*lppData) + size; 

	return ERROR_SUCCESS;
}

//  
//  FUNCTION:     DWORD Close()
//
//  PARAMETERS:   
//
//  RETURN VALUE: 0
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:      
//
//  COMMENTS:     
//
extern "C" __declspec(dllexport) DWORD WINAPI Close()
{
	if (counter.CounterNameTitleIndex == 0)
	{
		return 0;
	}

	fnClose();
	FreeLibrary(hOriginal);
	hOriginal = 0;
	counter.CounterNameTitleIndex = 0;
	return 0;
}

/////////////////////////////////////
//
//  
//
/////////////////////////////////////

HINSTANCE hInst = 0;

STDAPI DllRegisterServer()
{
	return Registry::DllRegister((HMODULE)hInst);
}

STDAPI DllUnregisterServer()
{
	return Registry::DllUnregister();
}

BOOL APIENTRY DllMain(HINSTANCE hInstance, DWORD reason, LPVOID)
{
	if (reason == DLL_PROCESS_ATTACH)
	{
		hInst = hInstance;
	}

	return TRUE;
}

///////////////////////////////////////////////////////