//
//  Copyright (C) 2003,  . All Rights Reserved.
//
//  PROJECT:    Performance Counter Sample
//
//  MODULE:     registry.cpp
//
//  PURPOSE:     ,    
//                 .     ...
//              ,    .
//
//  AUTHOR(S):   
//
//  PLATFORMS:  Windows NT/2000/XP
//

#include <windows.h>
#include <TCHAR.H>
#include <math.h>
#include "registry.h"

namespace Registry
{
	const TCHAR* perfKey   = TEXT("System\\CurrentControlSet\\Services\\PerfOS\\Performance");
	const TCHAR* perfLib   = TEXT("Software\\Microsoft\\Windows NT\\CurrentVersion\\Perflib");

	const TCHAR* myName = TEXT("MyCoolCounter");
	const TCHAR* myHelp = TEXT("MyCoolHelp");

	const TCHAR* counter_name = TEXT("MyCoolCounter");
	const TCHAR* counter_help = TEXT("MyCoolHelp");


	int UpdateAllBases(HKEY hPerf, int name_index, int help_index, const TCHAR* name, const TCHAR* help);
	int AddTitleToBase(HKEY hKey, const TCHAR* value, int index, const TCHAR* str);
	int ClearTitleFromBase(HKEY hKey, const TCHAR* value, int index);

	int GetDWORD(HKEY hKey, const TCHAR* value, DWORD* data);
	int SetDWORD(HKEY hKey, const TCHAR* value, DWORD data);

	int GetString(HKEY hKey, const CHAR* value, CHAR* data);
	int SetString(HKEY hKey, const TCHAR* value, const TCHAR* data);

	int RenameValue(HKEY hKey, const TCHAR* from, const TCHAR* to);	

	bool IsExistValue(HKEY hKey, const TCHAR* value);
}

//////////////////////////////////////////////////////////////////////
//
//  
//
//////////////////////////////////////////////////////////////////////

//  
//  FUNCTION:     int DllRegister(HMODULE)
//
//  PARAMETERS:   [IN] hModule -  dll
//
//  RETURN VALUE:  
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:       dll   
//
//  COMMENTS:     
//
int Registry::DllRegister(HMODULE hModule)
{
	LONG res;
	HKEY hPerf;

	res = RegCreateKeyEx(
			HKEY_LOCAL_MACHINE,
			perfKey,
			0,
			0,
			0,
			KEY_SET_VALUE | KEY_QUERY_VALUE,
			NULL,
			&hPerf,
			NULL);

	if (res != ERROR_SUCCESS) 
	{
		return res;
	}

	if (IsExistValue(hPerf, TEXT("old_Library")))
	{
		RegCloseKey(hPerf);
		return ERROR_ALREADY_EXISTS;
	}

	//   
	RenameValue(hPerf, TEXT("Library"), TEXT("old_Library"));
	RenameValue(hPerf, TEXT("Open"),    TEXT("old_Open"));
	RenameValue(hPerf, TEXT("Close"),   TEXT("old_Close"));
	RenameValue(hPerf, TEXT("Collect"), TEXT("old_Collect"));

	TCHAR str[_MAX_PATH];
	GetModuleFileName(hModule, str, sizeof(str)/sizeof(str[0]));

	//  
	SetString(hPerf, TEXT("Library"), str); 
	SetString(hPerf, TEXT("Open"),    TEXT("Open")); 
	SetString(hPerf, TEXT("Close"),   TEXT("Close")); 
	SetString(hPerf, TEXT("Collect"), TEXT("Collect")); 

	HKEY hPerfLib;	

	//  Perflib
	res = RegCreateKeyEx(
			HKEY_LOCAL_MACHINE,
			perfLib,
			0,
			0,
			0,
			KEY_ENUMERATE_SUB_KEYS | KEY_SET_VALUE | KEY_QUERY_VALUE,
			NULL,
			&hPerfLib,
			NULL);

	DWORD name_index;
	DWORD help_index;

	//    
	GetDWORD(hPerfLib, TEXT("Last Counter"), &name_index);
	GetDWORD(hPerfLib, TEXT("Last Help"),    &help_index);

	//  
	name_index += 2;
	help_index += 2;

	//     
	UpdateAllBases(hPerfLib, name_index, help_index, counter_name, counter_help);

	//    
	SetDWORD(hPerfLib, TEXT("Last Counter"), name_index);
	SetDWORD(hPerfLib, TEXT("Last Help"),    help_index);

	RegCloseKey(hPerfLib);

	//     
	SetDWORD(hPerf, myName, name_index);
	SetDWORD(hPerf, myHelp, help_index);	

	RegCloseKey(hPerf);

	return ERROR_SUCCESS;
}

//  
//  FUNCTION:     int DllUnregister()
//
//  PARAMETERS:   
//
//  RETURN VALUE:  
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:          dll 
//
//  COMMENTS:     
//
int Registry::DllUnregister()
{
	LONG res;
	HKEY hPerf;

	//   
	res = RegCreateKeyEx(
			HKEY_LOCAL_MACHINE,
			perfKey,
			0,
			0,
			0,
			KEY_SET_VALUE | KEY_QUERY_VALUE,
			NULL,
			&hPerf,
			NULL);

	if (res != ERROR_SUCCESS) 
	{
		return res;
	}

	//   
	RenameValue(hPerf, TEXT("old_Library"), TEXT("Library"));
	RenameValue(hPerf, TEXT("old_Open"),    TEXT("Open"));
	RenameValue(hPerf, TEXT("old_Close"),   TEXT("Close"));
	RenameValue(hPerf, TEXT("old_Collect"), TEXT("Collect"));

	DWORD name_index;
	DWORD help_index;

	//     
	GetDWORD(hPerf, myName, &name_index);
	GetDWORD(hPerf, myHelp, &help_index);

	//   
	RegDeleteValue(hPerf, myName);
	RegDeleteValue(hPerf, myHelp);

	RegCloseKey(hPerf);

	HKEY hPerfLib;	

	//  Perflib
	res = RegCreateKeyEx(
			HKEY_LOCAL_MACHINE,
			perfLib,
			0,
			0,
			0,
			KEY_ENUMERATE_SUB_KEYS | KEY_SET_VALUE | KEY_QUERY_VALUE,
			NULL,
			&hPerfLib,
			NULL);
	
	//    
	UpdateAllBases(hPerf, name_index, help_index, NULL, NULL);

	DWORD last_name_index;
	DWORD last_help_index;

	//    
	GetDWORD(hPerfLib, TEXT("Last Counter"), &last_name_index);
	GetDWORD(hPerfLib, TEXT("Last Help"),    &last_help_index);

	if (last_name_index == name_index)
	{
		//  
		SetDWORD(hPerfLib, TEXT("Last Counter"), last_name_index - 2);
	}

	if (last_help_index == help_index)
	{
		//  
		SetDWORD(hPerfLib, TEXT("Last Help"), last_help_index - 2);
	}

	RegCloseKey(hPerfLib);

	return ERROR_SUCCESS;
}

//  
//  FUNCTION:     int GetMy(DWORD* name, DWORD* help)
//
//  PARAMETERS:   [OUT] name -   
//                [OUT] help -   
//
//  RETURN VALUE:  
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:           . 
//
//  COMMENTS:     
//
int Registry::GetMy(DWORD* name, DWORD* help)
{
	LONG res;
	HKEY hPerf;

	res = RegOpenKeyEx(
			HKEY_LOCAL_MACHINE,
			perfKey,
			0,
			KEY_QUERY_VALUE,
			&hPerf);

	if (res != ERROR_SUCCESS) 
	{
		return res;
	}

	res = GetDWORD(hPerf, myName, name);

	if (res != ERROR_SUCCESS) 
	{
		RegCloseKey(hPerf);
		return res;
	}

	res = GetDWORD(hPerf, myHelp, help);

	RegCloseKey(hPerf);

	if (res != ERROR_SUCCESS) 
	{
		return res;
	}

	return ERROR_SUCCESS;
}

//  
//  FUNCTION:     int GetOriginalDll(CHAR*, CHAR*, CHAR*, CHAR*)
//
//  PARAMETERS:   [OUT] path    -   dll
//                [OUT] open    -   Open
//                [OUT] close   -   Close
//                [OUT] collect -   Collect
//
//  RETURN VALUE:  
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:         dll. 
//
//  COMMENTS:     
//
int Registry::GetOriginalDll(CHAR* path, CHAR* open, CHAR* close, CHAR* collect)
{
	LONG res;
	HKEY hPerf;

	res = RegOpenKeyExA(
			HKEY_LOCAL_MACHINE,
			perfKey,
			0,
			KEY_QUERY_VALUE,
			&hPerf);

	GetString(hPerf, "old_Library", path);
	GetString(hPerf, "old_Open",    open);
	GetString(hPerf, "old_Close",   close);
	GetString(hPerf, "old_Collect", collect);

	RegCloseKey(hPerf);

	return ERROR_SUCCESS;
}

//////////////////////////////////////////////////////////////
//
//  
//
//////////////////////////////////////////////////////////////

//  
//  FUNCTION:     int UpdateAllBases(HKEY, int, int, const TCHAR*, const TCHAR*)
//
//  PARAMETERS:   [IN] hKey       -  
//                [IN] name_index -  
//                [IN] help_index -  
//                [IN] name       -   NULL
//                [IN] help       -   NULL
//
//  RETURN VALUE:  
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:         . 
//
//  COMMENTS:     
//
int Registry::UpdateAllBases(
						HKEY hPerf, 
						int name_index, int help_index, 
						const TCHAR* name, const TCHAR* help)
{
	TCHAR key_name[4];

	for (int i = 0;; i++)
	{
		int res = RegEnumKey(hPerf, i, key_name, sizeof(key_name) / sizeof(key_name[0]));

		if (res != ERROR_SUCCESS)
		{
			break;
		}

		HKEY hKey;

		RegCreateKeyEx(
			hPerf,
			key_name,
			0,
			0,
			0,
			KEY_SET_VALUE | KEY_QUERY_VALUE,
			NULL,
			&hKey,
			NULL);

		if (name == NULL)
		{
			ClearTitleFromBase(hKey, TEXT("Counter"), name_index);
			ClearTitleFromBase(hKey, TEXT("Help"),    help_index);
		}
		else
		{
			AddTitleToBase(hKey, TEXT("Counter"), name_index, name);
			AddTitleToBase(hKey, TEXT("Help"),    help_index, help);
		}

		RegCloseKey(hKey);
	}

	return 0;
}

//  
//  FUNCTION:     int AddTitleToBase(HKEY, const TCHAR*, int)
//
//  PARAMETERS:   [IN] hKey  -  
//                [IN] value -  c  
//                [IN] index -  
//                [IN] str   - 
//
//  RETURN VALUE:  
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:               . 
//
//  COMMENTS:     
//
int Registry::AddTitleToBase(HKEY hKey, const TCHAR* value, int index, const TCHAR* str)
{
	int res;
	DWORD size;

	//  
	res = RegQueryValueEx(hKey, value, 0, 0, 0, &size);

	if (res != ERROR_SUCCESS) 
	{
		return res;
	}

	//   -  ,  0  ,  ,  0  
	DWORD add_size = (log10(index) + 1 + 1 + lstrlen(str) + 1) * sizeof(TCHAR);

	BYTE* data = (BYTE*)malloc(size + add_size);

	if (!data)
	{
		return ERROR_OUTOFMEMORY;
	}

	//  
	res = RegQueryValueEx(hKey, value, 0, 0, data, &size);

	if (res != ERROR_SUCCESS) 
	{
		free(data);
		return res;
	}

	//   -  0,   
	TCHAR* tmp = (TCHAR*)(data + size - sizeof(TCHAR));

	//  
	wsprintf(tmp, TEXT("%d"), index);
	
	TCHAR* tmp2 = tmp + lstrlen(tmp) + 1;
	
	//  
	lstrcpy(tmp2, str);

	//   0
	tmp2[lstrlen(tmp2) + 1] = 0;

	res = RegSetValueEx(hKey, value, 0, REG_MULTI_SZ, data, size + add_size);

	free(data);

	return res;
}

//  
//  FUNCTION:     int ClearTitleFromBase(HKEY, const TCHAR*, int)
//
//  PARAMETERS:   [IN] hKey  -  
//                [IN] value -  c  
//                [IN] index -  
//
//  RETURN VALUE:  
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:              
//                 . 
//
//  COMMENTS:     
//
int Registry::ClearTitleFromBase(HKEY hKey, const TCHAR* value, int index)
{
	int res;
	DWORD size;

	//   
	res = RegQueryValueEx(hKey, value, 0, 0, 0, &size);

	if (res != ERROR_SUCCESS) 
	{
		return res;
	}

	BYTE* data = (BYTE*)malloc(size);

	if (!data)
	{
		return ERROR_OUTOFMEMORY;
	}

	//  
	res = RegQueryValueEx(hKey, value, 0, 0, data, &size);

	if (res != ERROR_SUCCESS) 
	{
		free(data);
		return res;
	}

	TCHAR* tmp = (TCHAR*)data;
	
	for (;;)
	{
		int len = lstrlen(tmp);

		if (len == 0)
		{
			//   
			break;
		}

		len++;

		len += lstrlen(tmp + len);

		len++;

		if (_ttoi(tmp) == index)
		{
			//  
			size -= len * sizeof(TCHAR);
			memmove(tmp, tmp + len, size - ((BYTE*)tmp - data));
			
			break;
		}

		tmp += len;
	}

	res = RegSetValueEx(hKey, value, 0, REG_MULTI_SZ, data, size);

	free(data);

	return res;
}

/////////////////////////////////////////////////////////////
//
//        
//
/////////////////////////////////////////////////////////////

//  
//  FUNCTION:     int GetDWORD(HKEY , const TCHAR* , DWORD* )
//
//  PARAMETERS:   [IN]  hKey   -  
//                [IN]  value  - 
//                [OUT] data   -  
//
//  RETURN VALUE:  
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:           DWORD. 
//
//  COMMENTS:     
//
int Registry::GetDWORD(HKEY hKey, const TCHAR* value, DWORD* data)
{
	LONG res;
	DWORD type = REG_DWORD;
	DWORD size = sizeof(DWORD);

	res = RegQueryValueEx(hKey, value, 0, &type, (BYTE*)data, &size);

	if (res != ERROR_SUCCESS) 
	{
		return res;
	}

	if (type != REG_DWORD)
	{
		return ERROR_INVALID_DATA;
	}

	return ERROR_SUCCESS;
}

//  
//  FUNCTION:     int SetDWORD(HKEY , const TCHAR* , DWORD)
//
//  PARAMETERS:   [IN] hKey   -  
//                [IN] value  - 
//                [IN] data   -  
//
//  RETURN VALUE:  
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:           DWORD. 
//
//  COMMENTS:     
//
int Registry::SetDWORD(HKEY hKey, const TCHAR* value, DWORD data)
{
	return RegSetValueEx(
				hKey, 
				value, 
				0, 
				REG_DWORD, 
				(const BYTE*)&data, 
				sizeof(data));
}

//  
//  FUNCTION:     int SetString(HKEY , const TCHAR* , const TCHAR* )
//
//  PARAMETERS:   [IN] hKey   -  
//                [IN] value  - 
//                [IN] data   -  
//
//  RETURN VALUE:  
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:          . 
//
//  COMMENTS:     
//
int Registry::SetString(HKEY hKey, const TCHAR* value, const TCHAR* data)
{
	return RegSetValueEx(
				hKey, 
				value, 
				0, 
				REG_SZ, 
				(const BYTE*)data, 
				(lstrlen(data) + 1) * sizeof(data[0]));
}

//  
//  FUNCTION:     int GetString(HKEY , const CHAR* , CHAR* )
//
//  PARAMETERS:   [IN]  hKey   -  
//                [IN]  value  - 
//                [OUT] data   -  
//
//  RETURN VALUE:  
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:          . 
//
//  COMMENTS:      ANSI 
//                ,     
//                 ...     ? 
//
int Registry::GetString(HKEY hKey, const CHAR* value, CHAR* data)
{
	DWORD size;
	DWORD type;
	int res = RegQueryValueExA(hKey, value, 0, &type, 0, &size);

	if (res != ERROR_SUCCESS)
	{
		return res;
	}

	if (type != REG_SZ)
	{
		return ERROR_INVALID_DATA;
	}

	//  ,  
	return RegQueryValueExA(
				hKey, 
				value, 
				0, 
				0, 
				(BYTE*)data, 
				&size);
}


//  
//  FUNCTION:     int RenameValue(HKEY, const TCHAR*, const TCHAR*)
//
//  PARAMETERS:   [IN] hKey -  
//                [IN] from -   
//                [IN] to   -   
//
//  RETURN VALUE:  
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:         . 
//
//  COMMENTS:     
// 
int Registry::RenameValue(HKEY hKey, const TCHAR* from, const TCHAR* to)
{
	int res;
	DWORD size;

	res = RegQueryValueEx(hKey, from, 0, 0, 0, &size);

	if (res != ERROR_SUCCESS) 
	{
		return res;
	}

	BYTE* data = (BYTE*)malloc(size);

	if (!data)
	{
		return ERROR_OUTOFMEMORY;
	}

	DWORD type;

	res = RegQueryValueEx(hKey, from, 0, &type, data, &size);

	if (res != ERROR_SUCCESS) 
	{
		free(data);
		return res;
	}

	res = RegSetValueEx(hKey, to, 0, type, data, size);

	free(data);

	if (res != ERROR_SUCCESS) 
	{
		return res;
	}

	RegDeleteValue(hKey, from);

	return ERROR_SUCCESS;
}

//  
//  FUNCTION:     bool IsExistValue(HKEY , const TCHAR* )
//
//  PARAMETERS:   [IN] hKey  -  
//                [IN] value -  
//
//  RETURN VALUE:  
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:      ,   
//
//  COMMENTS:     
// 
bool Registry::IsExistValue(HKEY hKey, const TCHAR* value)
{
	DWORD size;

	int res = RegQueryValueEx(hKey, value, 0, 0, 0, &size);

	if (res != ERROR_SUCCESS) 
	{
		return false;
	}

	return true;
}

/////////////////////////////////////////////////////////////