#include <windows.h>
#include "registry.h"

namespace Registry
{
	const TCHAR* perfKey   = TEXT("System\\CurrentControlSet\\Services\\MyPerf1\\Performance");
	const TCHAR* firstName = TEXT("First Counter");
	const TCHAR* firstHelp = TEXT("First Help");

	int GetDWORD(HKEY hKey, const TCHAR* value, DWORD* data);
	int SetString(HKEY hKey, const TCHAR* value, const TCHAR* data);
}

//////////////////////////////////////////////////////////////////////
//
//  
//
//////////////////////////////////////////////////////////////////////

int Registry::DllRegister(HMODULE hModule)
{
	LONG res;
	HKEY hPerf;

	res = RegCreateKeyEx(
			HKEY_LOCAL_MACHINE,
			perfKey,
			0,
			0,
			0,
			KEY_SET_VALUE,
			NULL,
			&hPerf,
			NULL);

	if (res != ERROR_SUCCESS) 
	{
		return res;
	}

	TCHAR str[_MAX_PATH];
	GetModuleFileName(hModule, str, sizeof(str)/sizeof(str[0]));

	SetString(hPerf, TEXT("Library"), str); 
	SetString(hPerf, TEXT("Open"),    TEXT("Open")); 
	SetString(hPerf, TEXT("Close"),   TEXT("Close")); 
	SetString(hPerf, TEXT("Collect"), TEXT("Collect")); 

	RegCloseKey(hPerf);

	return ERROR_SUCCESS;
}

int Registry::DllUnregister()
{
	RegDeleteKey(HKEY_LOCAL_MACHINE, perfKey);

	TCHAR str[_MAX_PATH];

	lstrcpy(str, perfKey);

	int i = lstrlen(str);

	while (str[i] != TEXT('\\')) i--;
	str[i] = 0;

	RegDeleteKey(HKEY_LOCAL_MACHINE, str);

	return ERROR_SUCCESS;
}

int Registry::GetFirst(DWORD* name, DWORD* help)
{
	LONG res;
	HKEY hPerf;

	res = RegOpenKeyEx(
			HKEY_LOCAL_MACHINE,
			perfKey,
			0,
			KEY_QUERY_VALUE,
			&hPerf);

	if (res != ERROR_SUCCESS) 
	{
		return res;
	}

	res = GetDWORD(hPerf, firstName, name);

	if (res != ERROR_SUCCESS) 
	{
		RegCloseKey(hPerf);
		return res;
	}

	res = GetDWORD(hPerf, firstHelp, help);

	RegCloseKey(hPerf);

	if (res != ERROR_SUCCESS) 
	{
		return res;
	}

	return ERROR_SUCCESS;
}

//////////////////////////////////////////////////////////////
//
//  
//
//////////////////////////////////////////////////////////////

int Registry::GetDWORD(HKEY hKey, const TCHAR* value, DWORD* data)
{
	LONG res;
	DWORD type = REG_DWORD;
	DWORD size = sizeof(DWORD);

	res = RegQueryValueEx(hKey, value, 0, &type, (BYTE*)data, &size);

	if (res != ERROR_SUCCESS) 
	{
		return res;
	}

	if (type != REG_DWORD)
	{
		return ERROR_INVALID_DATA;
	}

	return ERROR_SUCCESS;
}

int Registry::SetString(HKEY hKey, const TCHAR* value, const TCHAR* data)
{
	return RegSetValueEx(
				hKey, 
				value, 
				0, 
				REG_SZ, 
				(const BYTE*)data, 
				(lstrlen(data) + 1) * sizeof(data[0]));
}

/////////////////////////////////////////////////////////////