//
//  Copyright (C) 2003,  . All Rights Reserved.
//
//  PROJECT:    Performance Counter Sample
//
//  MODULE:     object_abstract.cpp
//
//  PURPOSE:       ,  ""  
//              "" .   ,
//                 
//
//  AUTHOR(S):   
//
//  PLATFORMS:  Windows NT/2000/XP
//

#include <windows.h>
#include "perf_objects.h"

namespace perf
{
	//
	//      8
	DWORD UpTo8(DWORD v)
	{
		if (v % 8)
		{
			v += 8;
			v -= v % 8;
		}

		return v;
	}
}

////////////////////////////////////////////////////////////////
//
// 
//
/////////////////////////////////////////////////////////////////

perf::object_type_abstract::object_type_abstract(DWORD offset) 
{
	_name_offset = offset;
	_buffer = NULL;
	_calculated = true;
}

////////////////////////////////////////////////////////////////
//
//   
//
/////////////////////////////////////////////////////////////////

//  
//  FUNCTION:     int get_counter_count()
//
//  PARAMETERS:   
//
//  RETURN VALUE:  
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:        
//
//  COMMENTS:     
// 
const int perf::object_type_abstract::get_counters_count() const
{
	return _counters.size();
}

//  
//  FUNCTION:     const perf::counter_definition& get_counter(int)
//
//  PARAMETERS:   [IN] i -  
//
//  RETURN VALUE: 
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:       
//
//  COMMENTS:     
// 
const perf::counter_definition& perf::object_type_abstract::get_counter(int i) const
{
	_ASSERT(i < _counters.size());
	_ASSERT(i >= 0);
	return _counters[i];
}

//  
//  FUNCTION:     void add_counter(DWORD, DWORD, DWORD)
//
//  PARAMETERS:   [IN] offset -     
//                [IN] type   - 
//                [IN] size   - 
//
//  RETURN VALUE: 
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:           
//
//  COMMENTS:           , 
//                     .
//
void perf::object_type_abstract::add_counter(DWORD offset, DWORD type, DWORD size)
{
	_counters.push_back(counter_definition(offset, type, size));

	if (_counters.size() == 1)
	{
		//  
		_counters[0].set_offset(sizeof(PERF_COUNTER_BLOCK));
		_calculated = true;
	}
	else if (_calculated)
	{
		//   ,     
		int index = _counters.size() - 1;
		_counters[index].set_offset(
							_counters[index - 1].get_offset() 
						  + _counters[index - 1].get_size());
	}
}

//  
//  FUNCTION:     void del_counter(int)
//
//  PARAMETERS:   [IN] i -  
//
//  RETURN VALUE: 
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:          
//
//  COMMENTS:         ,    
//
void perf::object_type_abstract::del_counter(int i)
{
	_ASSERT(i < _counters.size());
	_ASSERT(i >= 0);

	_counters.erase(_counters.begin() + i);

	if (_calculated && (i != _counters.size()))
	{
		//    
		_calculated = false;
	}
}

//  
//  FUNCTION:     void set_counter_size(int, DWORD)
//
//  PARAMETERS:   [IN] i    -  
//                [IN] size -  
//
//  RETURN VALUE: 
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:        
//
//  COMMENTS:         ,    
//
void perf::object_type_abstract::set_counter_size(int i, DWORD size)
{
	_ASSERT(i < _counters.size());
	_ASSERT(i >= 0);

	DWORD old_size = _counters[i].get_size();
	_counters[i].set_size(size);

	if ((old_size != _counters[i].get_size()) && (i != _counters.size() - 1))
	{
		//        -  
		_calculated = false;
	}
}

//  
//  FUNCTION:     void calculate_offsets()
//
//  PARAMETERS:   
//
//  RETURN VALUE: 
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:        
//
//  COMMENTS:     
//
void perf::object_type_abstract::calculate_offsets() const
{
	if (_calculated) return;

	int data_length = sizeof(PERF_COUNTER_BLOCK);

	for (int i = 0; i < _counters.size(); i++)
	{
		_counters[i].set_offset(data_length);
		data_length += _counters[i].get_size();
	}

	_calculated = true;
}

////////////////////////////////////////////////////////////////
//
//  
//
/////////////////////////////////////////////////////////////////

//  
//  FUNCTION:     int get_definition_length()
//
//  PARAMETERS:   
//
//  RETURN VALUE:   
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:         
//
//  COMMENTS:     
// 
int perf::object_type_abstract::get_definition_length() const
{
	return UpTo8(
			  sizeof(PERF_OBJECT_TYPE) 
			+ _counters.size() * counter_definition::size());
}

//  
//  FUNCTION:     int get_data_length()
//
//  PARAMETERS:   
//
//  RETURN VALUE:   
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:         
//
//  COMMENTS:          ,
//                   
// 
int perf::object_type_abstract::get_data_length() const
{
	int size = 0;

	if (!_calculated)
	{
		calculate_offsets();
	}

	if (_counters.size() > 0)
	{
		//       -   
		size += _counters[_counters.size() - 1].get_offset();
		size += _counters[_counters.size() - 1].get_size();
	}
	else
	{
		//       PERF_COUNTER_BLOCK
		size += sizeof(PERF_COUNTER_BLOCK);
	}

	return UpTo8(size);
}

//////////////////////////////////////////////////////////////
//
//   
//
//////////////////////////////////////////////////////////////

//  
//  FUNCTION:     void set_buffer(void*)
//
//  PARAMETERS:   [IN] buffer - 
//
//  RETURN VALUE: 
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:       ,  
//
//  COMMENTS:     
// 
void perf::object_type_abstract::set_buffer(void* buffer)
{
	_buffer = buffer;
}

//  
//  FUNCTION:     void write_header()
//
//  PARAMETERS:   
//
//  RETURN VALUE: 
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:       , ..  PERF_OBJECT_TYPE,
//                  PERF_COUNTER_DEFINITION  
//                PERF_COUNTER_BLOCK,   . 
//
//  COMMENTS:           
//                  ,    
//                   . 
// 
void perf::object_type_abstract::write_header()
{
	//  
	calculate_offsets();

	PERF_OBJECT_TYPE type = {0};

	//  type  
	fill_structure(&type);

	//   
	write_buffer(0, &type, sizeof(type));

	PERF_COUNTER_DEFINITION counter = {0};

	int offset = type.HeaderLength;

	for (int i = 0; i < get_counters_count(); i++)
	{
		//  counter     i
		get_counter(i).fill_structure(&counter);

		//   
		write_buffer(offset, &counter, sizeof(counter));

		offset += counter.ByteLength;
	}

	if (get_instances_count() == PERF_NO_INSTANCES)
	{
		//   
		PERF_COUNTER_BLOCK bl = {get_data_length()};

		//    
		write_buffer(get_definition_length(), &bl, sizeof(bl));
	}
}

//////////////////////////////////////////////////////////////
//
//  
//
////////////////////////////////////////////////////////////////

//  
//  FUNCTION:     int get_counter_data_offset(int)
//
//  PARAMETERS:   [IN] i -  
//
//  RETURN VALUE:       
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:            
//                 ,     ,
//                 .
//
//  COMMENTS:     
// 
int perf::object_type_abstract::get_counter_data_offset(int i) const
{
	_ASSERT(i < _counters.size());
	_ASSERT(i >= 0);

	if (!_calculated)
	{
		calculate_offsets();
	}

	return _counters[i].get_offset();
}

//  
//  FUNCTION:     void fill_structure(PERF_OBJECT_TYPE*)
//
//  PARAMETERS:   [OUT] pot - 
//
//  RETURN VALUE: 
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:         
//
//  COMMENTS:     
// 
void perf::object_type_abstract::fill_structure(PERF_OBJECT_TYPE* pot) const
{
	ZeroMemory(pot, sizeof(*pot));

	pot->ObjectNameTitleIndex = base_name_index + get_name_offset();
	pot->ObjectHelpTitleIndex = base_help_index + get_name_offset();
	pot->NumCounters          = get_counters_count();
	pot->HeaderLength         = sizeof(*pot);
	pot->DefinitionLength     = get_definition_length();
	pot->NumInstances         = get_instances_count(); //  
	pot->TotalByteLength      = get_total_length();    //  
}

/////////////////////////////////////////////////////////