//
//  Copyright (C) 2003,  . All Rights Reserved.
//
//  PROJECT:    Performance Counter Sample
//
//  MODULE:     object_instance.cpp
//
//  PURPOSE:    ,  "" .  
//                     
//               
//
//  AUTHOR(S):   
//
//  PLATFORMS:  Windows NT/2000/XP
//

#include <windows.h>
#include "perf_objects.h"

////////////////////////////////////////////////////////////////////
//
// 
//
////////////////////////////////////////////////////////////////////

//  
//  FUNCTION:     int get_instance_with_out_name_length()
//
//  PARAMETERS:   
//
//  RETURN VALUE:         
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:              
//
//  COMMENTS:          
// 
int perf::object_type_instance::get_instance_with_out_name_length() const
{
	int size = get_data_length() + sizeof(PERF_INSTANCE_DEFINITION);

	return UpTo8(size);
}

//  
//  FUNCTION:     int get_next_length(DWORD)
//
//  PARAMETERS:   [IN] id   - ,  
//
//  RETURN VALUE:  ,       
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:        ,       
//
//  COMMENTS:     
// 
int perf::object_type_instance::get_next_length(DWORD) const
{
	int length = get_total_length();

	length += UpTo8(sizeof(PERF_INSTANCE_DEFINITION));
	length += get_data_length();

	return length;
}

//  
//  FUNCTION:     int get_next_length(const WCHAR*)
//
//  PARAMETERS:   [IN] name - 
//
//  RETURN VALUE:  ,       
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:        ,       
//
//  COMMENTS:     
// 
int perf::object_type_instance::get_next_length(const WCHAR* name) const
{
	int length = get_total_length();

	length += sizeof(PERF_INSTANCE_DEFINITION);

	if (name)
	{
		length += (lstrlenW(name) + 1) * sizeof(WCHAR);
	}
	else
	{
		//   -  NULL-
		length += sizeof(WCHAR);
	}

	length = UpTo8(length);

	length += get_data_length();

	return length;
}

//  
//  FUNCTION:     int get_total_length()
//
//  PARAMETERS:   
//
//  RETURN VALUE:      
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:            
//
//  COMMENTS:     
// 
int perf::object_type_instance::get_total_length() const
{
	if (_last.index < 0)
	{
		return get_definition_length();
	}

	const PERF_INSTANCE_DEFINITION* pid;
	
	pid = (const PERF_INSTANCE_DEFINITION*)read_buffer(get_definition_length() + _last.offset);

	return get_definition_length() + _last.offset + pid->ByteLength + get_data_length();
}

//  
//  FUNCTION:     int get_instances_count()
//
//  PARAMETERS:   
//
//  RETURN VALUE:   
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:      
//
//  COMMENTS:     
// 
int perf::object_type_instance::get_instances_count() const
{
	if (_last.index < 0) return 0;
	return _last.index + 1;
}

//////////////////////////////////////////////////////////////
//
//   
//
//////////////////////////////////////////////////////////////

//  
//  FUNCTION:     void clear_all_instances()
//
//  PARAMETERS:   
//
//  RETURN VALUE: 
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:           
//                 
//
//  COMMENTS:           
// 
void perf::object_type_instance::clear_all_instances()
{
	_last.index = -1;
	_last.offset = 0;
	_last_use.index = 0;
	_last_use.offset = 0;
}

//  
//  FUNCTION:     void write_instance(DWORD, DWORD, DWORD)
//
//  PARAMETERS:   [IN] id              -  
//                [IN] parent_index    -   - 
//                [IN] parent_instance -   
//
//  RETURN VALUE: 
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:               
//
//  COMMENTS:     
// 
void perf::object_type_instance::write_instance(DWORD id, DWORD parent_index, DWORD parent_instance)
{
	int offset = get_total_length();

	PERF_INSTANCE_DEFINITION instance = {0};

	instance.ByteLength = UpTo8(sizeof(instance));
	instance.UniqueID   = id;
	instance.NameLength = 0;
	instance.NameOffset = 0;
	instance.ParentObjectTitleIndex = parent_index;
	instance.ParentObjectInstance   = parent_instance;

	write_buffer(offset, &instance, sizeof(instance));

	PERF_COUNTER_BLOCK bl = {get_data_length()};

	write_buffer(offset + instance.ByteLength, &bl, sizeof(bl)); 

	_last.index++;
	_last.offset = offset - get_definition_length();
}

//  
//  FUNCTION:     void write_instance(const WCHAR*, DWORD, DWORD)
//
//  PARAMETERS:   [IN] name            -  
//                [IN] parent_index    -   - 
//                [IN] parent_instance -   
//
//  RETURN VALUE: 
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:               
//
//  COMMENTS:     
// 
void perf::object_type_instance::write_instance(const WCHAR* name, DWORD parent_index, DWORD parent_instance)
{
	int offset = get_total_length();

	int name_length = 0;

	if (name) name_length = lstrlenW(name) * sizeof(WCHAR);

	PERF_INSTANCE_DEFINITION instance = {0};

	instance.ByteLength = UpTo8(sizeof(instance) + name_length + sizeof(WCHAR));
	instance.UniqueID   = PERF_NO_UNIQUE_ID;
	instance.NameLength = name_length + sizeof(WCHAR);
	instance.NameOffset = sizeof(instance);
	instance.ParentObjectTitleIndex = parent_index;
	instance.ParentObjectInstance   = parent_instance;

	write_buffer(offset, &instance, sizeof(instance));

	if (name)
	{
		write_buffer(offset + sizeof(instance), name, name_length);
	}

	write_buffer(offset + sizeof(instance) + name_length, L"", sizeof(WCHAR));

	PERF_COUNTER_BLOCK bl = {get_data_length()};

	write_buffer(offset + instance.ByteLength, &bl, sizeof(bl)); 

	_last.index++;
	_last.offset = offset - get_definition_length();
}

//  
//  FUNCTION:     void write_counter(int, int, DWORD)
//
//  PARAMETERS:   [IN] inst  -  
//                [IN] i     -  
//                [IN] value - 
//
//  RETURN VALUE: 
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:           
//
//  COMMENTS:        
// 
void perf::object_type_instance::write_counter(int inst, int i, DWORD value)
{
	int offset = get_definition_length() + get_instance_counter_data_offset(inst, i);

	write_buffer(offset, &value, sizeof(value));	
}

//  
//  FUNCTION:     void write_counter(int, int, const LARGE_INTEGER&)
//
//  PARAMETERS:   [IN] inst  -  
//                [IN] i     -  
//                [IN] value - 
//
//  RETURN VALUE: 
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:           
//
//  COMMENTS:        
// 
void perf::object_type_instance::write_counter(int inst, int i, const LARGE_INTEGER& value)
{
	int offset = get_definition_length() + get_instance_counter_data_offset(inst, i);

	write_buffer(offset, &value, sizeof(value));
}

//  
//  FUNCTION:     void write_counter(int, int, const void*, int)
//
//  PARAMETERS:   [IN] inst  -  
//                [IN] i     -  
//                [IN] value - 
//                [IN] size  - 
//
//  RETURN VALUE: 
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:           
//
//  COMMENTS:        
// 
void perf::object_type_instance::write_counter(int inst, int i, const void* value, int size)
{
	int offset = get_definition_length() + get_instance_counter_data_offset(inst, i);

	write_buffer(offset, value, size);
}

//  
//  FUNCTION:     void write_last_counter(int, DWORD)
//
//  PARAMETERS:   [IN] i     -  
//                [IN] value - 
//
//  RETURN VALUE: 
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:           
//
//  COMMENTS:        
// 
void perf::object_type_instance::write_last_counter(int i, DWORD value)
{
	int offset = get_definition_length() + get_instance_counter_data_offset(_last.index, i);

	write_buffer(offset, &value, sizeof(value));	
}

//  
//  FUNCTION:     void write_last_counter(int, const LARGE_INTEGER&)
//
//  PARAMETERS:   [IN] i     -  
//                [IN] value - 
//
//  RETURN VALUE: 
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:           
//
//  COMMENTS:        
// 
void perf::object_type_instance::write_last_counter(int i, const LARGE_INTEGER& value)
{
	int offset = get_definition_length() + get_instance_counter_data_offset(_last.index, i);

	write_buffer(offset, &value, sizeof(value));
}

//  
//  FUNCTION:     void write_last_counter(int, const void*, int)
//
//  PARAMETERS:   [IN] i     -  
//                [IN] value - 
//                [IN] size  - 
//
//  RETURN VALUE: 
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:           
//
//  COMMENTS:        
// 
void perf::object_type_instance::write_last_counter(int i, const void* value, int size)
{
	int offset = get_definition_length() + get_instance_counter_data_offset(_last.index, i);

	write_buffer(offset, value, size);
}

//////////////////////////////////////////////////////////////////
//
//  
//
///////////////////////////////////////////////////////////////////

//  
//  FUNCTION:     int get_instance_offset(int)
//
//  PARAMETERS:   [IN] i -  
//
//  RETURN VALUE:      
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:            
//
//  COMMENTS:        
// 
int perf::object_type_instance::get_instance_offset(int i) const
{
	_ASSERT(i <= _last.index);
	_ASSERT(i >= 0);

	if (i == _last.index)
	{
		//  
		return _last.offset;
	}

	if (i == _last_use.index)
	{
		//   
		return _last_use.offset;
	}

	int index  = 0;
	int offset = 0;

	if ((_last_use.index < i) && (_last_use.index >= 0))
	{
		//     ,  
		//    .
		index  = _last_use.index;
		offset = _last_use.offset;
	}
	
	for (; index < i; index++)
	{
		const PERF_INSTANCE_DEFINITION* pid;
		pid = (const PERF_INSTANCE_DEFINITION*)read_buffer(get_definition_length() + offset);

		offset += pid->ByteLength;

		const PERF_COUNTER_BLOCK* pcb;
		pcb = (const PERF_COUNTER_BLOCK*)read_buffer(get_definition_length() + offset);

		offset += pcb->ByteLength;
	}

	_last_use.index  = i;
	_last_use.offset = offset;

	return offset;
}

//  
//  FUNCTION:     int get_instance_counter_data_offset(int, int)
//
//  PARAMETERS:   [IN] inst -  
//                [IN] i    -  
//
//  RETURN VALUE:      
//
//  REVISION:     11.08.2003   
//
//  PURPOSE:            
//
//  COMMENTS:     
// 
int perf::object_type_instance::get_instance_counter_data_offset(int inst, int i) const
{
	//    
	int offset = get_instance_offset(inst);

	const PERF_INSTANCE_DEFINITION* pid;
	pid = (const PERF_INSTANCE_DEFINITION*)read_buffer(get_definition_length() + offset);

	//     
	offset += pid->ByteLength;

	//    
	offset += get_counter_data_offset(i);

	return offset;
}

/////////////////////////////////////////////////////////